/**
 * @file lv_console.c
 *
 */

/*********************
 *      INCLUDES
 *********************/
#include "lv_terminal.h"
#include "lv_console.h"
#include "lv_page.h"	
#include <stdio.h>
#include "lv_keyboard_conf.h"

/*********************
 *      DEFINES
 *********************/
#define LV_OBJX_NAME "lv_console"

/**********************
 *      TYPEDEFS
 **********************/

/**********************
 *  STATIC PROTOTYPES
 **********************/
static lv_res_t lv_console_signal(lv_obj_t * cb, lv_signal_t sign, void * param);
static lv_style_list_t * lv_console_get_style(lv_obj_t * cb, uint8_t type);
static void kb_event_cb(lv_obj_t * _kb, lv_event_t e);

/**********************
 *  STATIC VARIABLES
 **********************/
static lv_signal_cb_t ancestor_signal;

/**********************
 *      MACROS
 **********************/

/**********************
 *   GLOBAL FUNCTIONS
 **********************/

/**
 * Create a console objects
 * @param par pointer to an object, it will be the parent of the new check box
 * @param copy pointer to a check box object, if not NULL then the new object will be copied from it
 * @return pointer to the created check box
 */
lv_obj_t * lv_console_create(lv_obj_t * par, const lv_obj_t * copy)
{
    LV_LOG_TRACE("console create started");
    if (copy != NULL) {
        LV_LOG_ERROR("console does not support copying");
        return NULL;
	}

    /*Create the ancestor basic object*/
    lv_obj_t * cs = lv_cont_create(par, copy);
    LV_ASSERT_MEM(cs);
    if(cs == NULL) return NULL;

    if(ancestor_signal == NULL) ancestor_signal = lv_obj_get_signal_cb(cs);

    lv_console_ext_t * ext = lv_obj_allocate_ext_attr(cs, sizeof(lv_console_ext_t));
    LV_ASSERT_MEM(ext);
    if(ext == NULL) {
        lv_obj_del(cs);
        return NULL;
    }

    ext->label  = NULL;

    lv_obj_set_signal_cb(cs, lv_console_signal);

    /*Init the new console object*/
    
    lv_coord_t w;
    lv_coord_t h;
    if(par) {
        w = lv_obj_get_width_fit(lv_obj_get_parent(cs));
        h = lv_obj_get_height_fit(lv_obj_get_parent(cs));
    }
    else {
        w = lv_disp_get_hor_res(NULL);
        h = lv_disp_get_ver_res(NULL);
    }
    
    lv_obj_set_size(cs, w, h);
    
    ext->label = lv_label_create(cs, NULL);
    
    ext->keyboard = lv_keyboard_create(cs, NULL);
    lv_terminal_keyboard_configure(ext->keyboard);
    lv_obj_set_event_cb(ext->keyboard, kb_event_cb);
    lv_indev_wait_release(lv_indev_get_act());
    
    printf("\ncs pos before kbd resize: %dx%d\n", lv_obj_get_x(cs), lv_obj_get_y(cs));
    printf("cs size before kbd resize: %dx%d\n", lv_obj_get_width(cs), lv_obj_get_height(cs));
    printf("kbd pos before resize: %dx%d\n", lv_obj_get_x(cs), lv_obj_get_y(cs));
    printf("kbd size before resize: %dx%d\n", lv_obj_get_width(cs), lv_obj_get_height(cs));
    
    lv_obj_set_width(ext->keyboard, lv_obj_get_width(cs));
    
    printf("\ncs pos after kbd resize: %dx%d\n", lv_obj_get_x(cs), lv_obj_get_y(cs));
    printf("cs size after kbd resize: %dx%d\n", lv_obj_get_width(cs), lv_obj_get_height(cs));
    printf("kbd pos agter resize: %dx%d\n", lv_obj_get_x(cs), lv_obj_get_y(cs));
    printf("kbd size after resize: %dx%d\n", lv_obj_get_width(cs), lv_obj_get_height(cs));
    

    LV_LOG_INFO("check box created");

    return cs;
}

/**********************
 *   STATIC FUNCTIONS
 **********************/

/**
 * Signal function of the check box
 * @param cs pointer to a check box object
 * @param sign a signal type from lv_signal_t enum
 * @param param pointer to a signal specific variable
 * @return LV_RES_OK: the object is not deleted in the function; LV_RES_INV: the object is deleted
 */
static lv_res_t lv_console_signal(lv_obj_t * cb, lv_signal_t sign, void * param)
{
    lv_res_t res;
    if(sign == LV_SIGNAL_GET_STYLE) {
        return ancestor_signal(cb, sign, param);
    }

    /* Include the ancient signal function */
    res = ancestor_signal(cb, sign, param);
    if(res != LV_RES_OK) return res;
    if(sign == LV_SIGNAL_GET_TYPE) return lv_obj_handle_get_type_signal(param, LV_OBJX_NAME);

    return res;
}


static void kb_event_cb(lv_obj_t * _kb, lv_event_t e)
{
	if(e != LV_EVENT_VALUE_CHANGED) return;
    const char * txt = lv_btnmatrix_get_active_btn_text(_kb);

	printf("button is ");
	while(*txt)
		printf("%02x ", (unsigned int) *txt++);
	printf("\n");
	
    lv_keyboard_def_event_cb(_kb, e);    
}
